#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Aug 30 16:20:56 2021

@author: patrick
"""

import os
import sys
import re
import numpy as np
import pandas as pd
import geopandas as gpd
import descartes as dc
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.patches as patches
from matplotlib import lines
from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
from matplotlib import rc

rc('font',**{'family':'serif',
             'size' : 10})
rc('text', usetex=True)
rc('axes', titlesize = 10, labelsize = 10)

# Set working directory to path of the script
os.chdir(os.path.dirname(os.path.realpath(__file__)))

path_proj = os.path.join('..', 'workflow')
path_bin = os.path.abspath(os.path.join(path_proj, 'C_hgsm_nor_3d', 'bin'))
sys.path.append(path_bin)

import raster_support as rs


#%% Load data 

# Series definitions
path_seriesdef = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                              'input_data', 'cunitID_series_definition.csv')
series_def = pd.read_csv(path_seriesdef)

# Bore reference data 
path_ref = os.path.join(path_proj, 'B_prepare_data', 'data', 
                        'boredata_selection_2021_forprocessing_2021-08-30.csv')
ref_bores = pd.read_csv(path_ref)

# Surface points from bores
path_surfpoints = os.path.join(path_proj, 'C_hgsm_nor_3d',
                               'input_data', 'surfacepoints_CUnitID.csv')
surfpoints = pd.read_csv(path_surfpoints)
surfpoints = surfpoints.astype({'comment': str})

ref_sp = surfpoints.drop_duplicates('ID')

# Only used bore ref
is_base_ref = np.array(['BASE' in idval for idval in ref_sp.ID.values])
ref_usedbores = ref_sp.iloc[~is_base_ref]

# GPDN q-base data points
is_base_q = np.array(['BASE-C' in idval for idval in surfpoints.ID.values])
ref_qbase =surfpoints.iloc[is_base_q & (surfpoints.surface.values == 'qp/S_2')]

# qh-base inferred 
is_qh_inferred = surfpoints.is_GPDN_qh.values | surfpoints.is_LBEG_qh.values
ref_qh_inferred = surfpoints.iloc[is_qh_inferred]

# Norderney coastline
path_0m = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                         'shapes', 'nor_coastline_0masl.shp')
coast = gpd.read_file(path_0m)

path_mhw = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                         'shapes', 'nor_mhw-line_125cmasl.shp')
mhw = gpd.read_file(path_mhw)

# Model area HGSM 3D
path_hgsm = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                         'shapes', 'modelarea_hgsm_norderney_epsg25832.shp')
hgsm_area = gpd.read_file(path_hgsm)

# Topography
path_topo = os.path.join(path_proj, 'C_hgsm_nor_3d',
                         'raster', 'topography_hgsm_nor.tif')
topo_rs = rs.Raster(path_topo, get_array = True)
nodata = topo_rs.raster.profile['nodata']
topo = topo_rs.array[0]
topo[topo == nodata] = np.nan

# Spatial information from raster data
nx = topo_rs.raster.profile['width']
ny = topo_rs.raster.profile['height']

dx = topo_rs.raster.profile['transform'][0]
dy = topo_rs.raster.profile['transform'][4] # is negative

xmin = topo_rs.raster.profile['transform'][2]
ymax = topo_rs.raster.profile['transform'][5]

x = xmin + np.arange(0, nx) * dx + 0.5 * dx
y = ymax + np.arange(0, ny) * dy + 0.5 * dy

X, Y = np.meshgrid(x, y)

xmax = xmin + nx*dx
ymin = ymax + ny*dy


path_topoall = os.path.join(path_proj, 'A_preprocess_data', 'raw',
                            'bathymetrie', 'bathymetrie_east_frisian_islands_dx10m_dy10m_utm32n.tif')
topoall_rs = rs.Raster(path_topoall, get_array = True)
nodata = topoall_rs.raster.profile['nodata']
topoall = topoall_rs.array[0][400:2500, 3500:6000]
topoall[topoall == nodata] = np.nan

# Spatial information from raster data
nxall = topoall_rs.raster.profile['width']
nyall = topoall_rs.raster.profile['height']

dxall = topoall_rs.raster.profile['transform'][0]
dyall = topoall_rs.raster.profile['transform'][4] # is negative


xall = np.array([topoall_rs.raster.xy(0, k)[0] for k in range(3500, 6000)])
yall = np.array([topoall_rs.raster.xy(k, 0)[1] for k in range(400, 2500)])

Xall, Yall = np.meshgrid(xall, yall)

xminall = xall.min()
ymaxall = yall.min()
xmaxall = xall.max()
yminall = yall.max()

#%% Bore distribution and depth
cmap_topo = cm.get_cmap('bone', 20)

fig = plt.figure(0, figsize = (17.6/2.54, 20/2.54))
cax_im = fig.add_axes([0.53, 0.13, 0.45, 0.025])
cax_sc = fig.add_axes([0.53, 0.0, 0.45, 0.025])

ax = fig.add_axes([0, 0, 1, 0.8])
im = ax.pcolormesh(X, Y, topo, shading = 'nearest', cmap = cmap_topo,
                   vmin = -20, vmax = 20, alpha = 1, zorder = 0)
cbar_im = fig.colorbar(im, cax = cax_im, fraction = 0.1,
                       orientation = 'horizontal')
cbar_im.set_label('Surface elevation [m NHN]')

# Norderney area
xcoast, ycoast = coast['geometry'][0].xy
ax.plot(xcoast, ycoast, 'k', zorder = 0, linewidth = 1)

xmhw, ymhw = mhw['geometry'][0].xy
ax.plot(xmhw, ymhw, 'k--', zorder = 0, linewidth = 1)

# HGSM model area
hgsmpoly = dc.PolygonPatch(hgsm_area['geometry'][0], facecolor = 'none', edgecolor = 'k',
                           zorder = 0)
ax.add_patch(hgsmpoly)

# Selected GPDN-3D data
strat = ['tmim-tpl', 'tmiu', 'tolm-tolo', 'tolm-teoo']
marker = ['x', '+', '^', 'v']
for (name, mar) in zip(strat, marker):
    dat = surfpoints.iloc[surfpoints.surface.values == name]
    ax.scatter(dat.X, dat.Y, c = 'lightgrey', s = 8, marker = mar,
               label = name)

# Inferred qh-basis
sci = ax.scatter(ref_qh_inferred.X, ref_qh_inferred.Y, marker = 'X', s = 50,
                 linewidth = 0.5, edgecolor = 'k',
                 label = 'GPDN or LBEG\nqh-base')
sci.set_facecolor('w')

# GPDN q-base data
scqb = ax.scatter(ref_qbase.X[ref_qbase.is_GPDN_q], ref_qbase.Y[ref_qbase.is_GPDN_q],
                  marker = 's', c = 'k', s = 12, label = 'GPDN q-base used')
scqbn = ax.scatter(ref_qbase.X[~ref_qbase.is_GPDN_q], ref_qbase.Y[~ref_qbase.is_GPDN_q],
                   marker = 's', edgecolor = 'k', s = 12, label = 'GPDN q-base\nnot used')
scqbn.set_facecolor('none')

# Bore positions and depth
sc = ax.scatter(ref_bores.Xcoord, ref_bores.Ycoord, c = ref_bores.ZcoordE_mNN, s = 8, 
                vmin = -150, cmap = 'YlOrRd_r')
cbar_sc = fig.colorbar(sc, cax = cax_sc, fraction = 0.1,
                       orientation = 'horizontal')
cbar_sc.set_label('Total depth of bore [m NHN]')

# Bores used
scb = ax.scatter(ref_usedbores.X, ref_usedbores.Y, s = 12, edgecolor = 'k',
                 label = 'bore used')
scb.set_facecolor('none')
    
# North arrow
ax.arrow(xmax-5e2, ymax-9.5e2, 0, 0, head_width = 400, fc = 'black', ec = 'none',
         overhang = 0.2,
         zorder = 10)
ax.text(xmax-5e2, ymax-3.5e2, '$\\mathbf{N}$', ha = 'center')

# Scale bar
scalebar = AnchoredSizeBar(ax.transData, 1000, '1000 m', 'lower left',
                           frameon = False, size_vertical = 10, sep = 5,
                           bbox_to_anchor = [120, 690])
scalebar.zorder = 200
ax.add_artist(scalebar)

ax.text(0.01, 0.94, '(a)', transform = ax.transAxes)
                           
ax.set_xlim(xmin, xmax)
ax.set_ylim(ymin, ymax)

ax.set_yticks(np.arange(y[-1].round(-3)+5e2, y[0].round(-3), 2e3).astype(int))
ax.set_yticklabels(ax.get_yticks(), rotation = 90, verticalalignment = 'center')

ax.set_aspect(1)

leg = ax.legend(frameon = False, loc = 'lower left', bbox_to_anchor = (0.275, -0.79),
                ncol = 1, handletextpad = 0.1)

ax.add_artist(leg)

handles = [lines.Line2D([0], [0], color = 'k', linestyle = '-'),
           lines.Line2D([0], [0], color = 'k', linestyle = '--')]
ax.legend(handles, ['0 m NHN', 'MHW = 1.25 m NHN'], 
          loc = 'lower right', frameon = False,
          bbox_to_anchor = (1.01, -0.02))



# Bores outside model area =====
axs = fig.add_axes([-0.113, -0.02, 0.5, 0.2])

im = axs.pcolormesh(Xall, Yall, topoall, shading = 'nearest', cmap = cmap_topo,
                   vmin = -20, vmax = 20, alpha = 1, zorder = 0)

# Norderney area
xcoast, ycoast = coast['geometry'][0].xy
axs.plot(xcoast, ycoast, 'k', zorder = 0, linewidth = 0.5)

xmhw, ymhw = mhw['geometry'][0].xy
axs.plot(xmhw, ymhw, 'k--', zorder = 0, linewidth = 0.5)

# HGSM model area
hgsmpoly = dc.PolygonPatch(hgsm_area['geometry'][0], facecolor = 'none', edgecolor = 'k',
                           zorder = 0)
axs.add_patch(hgsmpoly)


is_out_qh = np.array(["outside model area" in com for com in ref_qh_inferred.comment.values])
# Inferred qh-basis
sci = axs.scatter(ref_qh_inferred.X[is_out_qh], ref_qh_inferred.Y[is_out_qh], marker = 'X', s = 50,
                 linewidth = 0.5, edgecolor = 'k',
                 label = 'GPDN or LBEG qh-base')
sci.set_facecolor('w')

# GPDN q-base data
is_out_qp = np.array(["outside model area" in com for com in ref_qbase.comment.values])
scqb = axs.scatter(ref_qbase.X[ref_qbase.is_GPDN_q & is_out_qp], ref_qbase.Y[ref_qbase.is_GPDN_q & is_out_qp],
                  marker = 's', c = 'k', s = 12, label = 'GPDN q-base used')
scqbn = axs.scatter(ref_qbase.X[~ref_qbase.is_GPDN_q & is_out_qp], ref_qbase.Y[~ref_qbase.is_GPDN_q & is_out_qp],
                   marker = 's', edgecolor = 'k', s = 12, label = 'GPDN q-base not used')
scqbn.set_facecolor('none')

# Bore positions and depth
is_bore_comment = np.array([
    re.search('[0-9][0-9][0-9][0-9]', com) is not None for com in surfpoints.comment.values
])
is_out_bores = np.array(["outside model area" in com for com in surfpoints.comment.values])
sc = axs.scatter(surfpoints.X[is_out_bores & is_bore_comment], 
                 surfpoints.Y[is_out_bores & is_bore_comment],
                 s = 8, facecolors = 'white', edgecolors = 'k')

# Bores used
scb = ax.scatter(ref_usedbores.X, ref_usedbores.Y, s = 12, edgecolor = 'k',
                 label = 'bore used')
scb.set_facecolor('none')

# Scale bar
scalebar = AnchoredSizeBar(axs.transData, 5000, '5 km', 'lower left',
                           frameon = False, size_vertical = 10, sep = 5,
                           bbox_to_anchor = [530, 120])
scalebar.zorder = 200
axs.add_artist(scalebar)

axs.text(0.03, 0.9, '(b)', transform = axs.transAxes)

axs.set_yticks([5945000, 5955000])
axs.set_yticklabels(axs.get_yticks(), rotation = 90, verticalalignment = 'center')

axs.set_aspect(1)

fig.savefig('Fig2.png', bbox_inches = 'tight', dpi = fig.dpi)

