#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct  3 14:04:31 2022

@author: patrick
"""

import os
import numpy as np
import pandas as pd
import geopandas as gpd
import matplotlib.pyplot as plt
from matplotlib import colors
from matplotlib import patches
from matplotlib import lines
from matplotlib import rc

rc('font',**{'family':'serif',
             'size' : 10})
rc('text', usetex=True)
rc('axes', titlesize = 10, labelsize = 10)

# Set working directory to path of the script
os.chdir(os.path.dirname(os.path.realpath(__file__)))

path_proj = os.path.join('..', 'workflow')



#%% Load data
# Norderney coastline
path_0m = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                       'shapes', 'nor_coastline_0masl.shp')
coast = gpd.read_file(path_0m)

path_mhw = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                        'shapes', 'nor_mhw-line_125cmasl.shp')
mhw = gpd.read_file(path_mhw)


# Bore profile interpretation data ----
# Bore interpretation definition file
# Contains UnitID names and corresponding colors.
path_boredef = os.path.join(path_proj, 'C_hgsm_nor_3d', 'input_data', 'unitID_definition.csv')
bore_def = pd.read_csv(path_boredef)
cmap_bores = colors.ListedColormap(bore_def.color.values)

# Interpreted bore profile data
path_boreprofiles = os.path.join(path_proj, 'C_hgsm_nor_3d', 'input_data', 'boredata_profiles_UnitID.csv')
boreprofiles = pd.read_csv(path_boreprofiles)

# North to south
west_id = {'2208HY0060': 0, '2208IG0009': 28, '2208HY0003': 59}
qh_lbeg_west = [-9, -9, -9] 
qh_gpdn_west = [-12, -18, -28]

# Make adaptions for plotting
boreprofiles.UnitID[
    (boreprofiles.ID.values == '2208HY0060') & (boreprofiles.Z.values == -9.91)
] = 'UNC_qhqp'
bore_def = pd.concat([
    bore_def, 
    pd.DataFrame({'UnitID': 'UNC_qhqp', 'color': '#feffb7'}, index = [0])
])

boreprofiles.UnitID[
    (boreprofiles.ID.values == '2208IG0009') & (boreprofiles.Z.values == -43.83)
] = 'UNC_qhqp'


# North to south
central_id = {'2209HY0040': 0, 'NyB19': 42, '2209HY0017': 69}

# Make adaptions for plotting
boreprofiles.UnitID[
    (boreprofiles.ID.values == 'NyB19') & (boreprofiles.Z.values == -8.72)
] = 'UNC_qhSTU'
bore_def = pd.concat([
    bore_def, 
    pd.DataFrame({'UnitID': 'UNC_qhSTU', 'color': '#feffb7'}, index = [0])
])

boreprofiles.UnitID[
    (boreprofiles.ID.values == '2209HY0017') & (boreprofiles.Z.values == -9.78)
] = 'UNC_qhSTU_lag'
boreprofiles.UnitID[
    (boreprofiles.ID.values == '2209HY0017') & (boreprofiles.Z.values == -24.78)
] = 'UNC_qhSTU_lag'
bore_def = pd.concat([
    bore_def, 
    pd.DataFrame({'UnitID': 'UNC_qhSTU_lag', 'color': '#feffb7'}, index = [0])
])

#%% Plot
plt.rcParams['hatch.linewidth'] = 4

fig = plt.figure(0, figsize = (17.6/2.54, 10/2.54))

addval = 10

# (a) western part =====
ax = fig.add_axes([0, 0, 0.3, 1])

for k, (bid, dist) in enumerate(west_id.items()):
    profile = boreprofiles.loc[boreprofiles.ID.values == bid]
    profile = profile.reset_index(drop = True)
    
    profile = pd.concat([pd.DataFrame(data = dict(profile.loc[0]), index = [0]), profile])
    profile = profile.reset_index(drop = True)
    profile.Z[0] += profile.Depth[0]
    profile = profile.assign(thickness = np.abs(profile.Z.diff()))
    
    profile_color = [np.where(layer == bore_def.UnitID.values)[0][0]\
                      for layer in profile.UnitID.values]
    
    for idx, layer in profile.iterrows():
        b = ax.bar(dist, bottom = layer.Z, height = layer.thickness, width = 2*addval,
                   facecolor = cmap_bores(profile_color[idx]), edgecolor = '#bdf2da')
        if layer.UnitID == 'UNC_qhqp':
            b.patches[0].set_hatch('//')
        ax.bar(dist, bottom = layer.Z, height = layer.thickness, width = 2*addval,
                   facecolor = 'none', edgecolor = 'k', zorder = 10)
        
    ax.plot([dist-addval, dist+addval], np.repeat(qh_gpdn_west[k], 2), 'k--')
    ax.plot([dist-addval, dist+addval], np.repeat(qh_lbeg_west[k], 2), 'k-.')

ax.annotate('', 
            xy = (west_id['2208HY0060'], 8),
            xytext = (west_id['2208IG0009'], 8),
            arrowprops = {'arrowstyle': '<->'},
            annotation_clip = False)
ax.text(0.5 * (west_id['2208IG0009'] - west_id['2208HY0060']) + west_id['2208HY0060'],
        9, f"{west_id['2208IG0009'] - west_id['2208HY0060']} m",
        horizontalalignment = 'center')

ax.annotate('', 
            xy = (west_id['2208IG0009'], 8),
            xytext = (west_id['2208HY0003'], 8),
            arrowprops = {'arrowstyle': '<->'},
            annotation_clip = False)
ax.text(0.5 * (west_id['2208HY0003'] - west_id['2208IG0009']) + west_id['2208IG0009'],
        9, f"{west_id['2208HY0003'] - west_id['2208IG0009']} m",
        horizontalalignment = 'center')

ax.set_ylim(-50, 12)

ax.text(ax.get_xlim()[0], ax.get_ylim()[1] + 1, '$\mathbf{N}$')
ax.text(ax.get_xlim()[1], ax.get_ylim()[1] + 1, '$\mathbf{S}$', 
        horizontalalignment = 'right')

ax.text(0.03, 0.96, '(a)', transform = ax.transAxes)

ax.xaxis.tick_top()
ax.set_xticks(list(west_id.values()))
ax.set_xticklabels(west_id.keys(), rotation = 45, horizontalalignment = 'left')

ax.set_ylabel('Elevation [m NHN]')

handles = [lines.Line2D([0], [0], color = 'k', linestyle = '--'),
           lines.Line2D([0], [0], color = 'k', linestyle = '-.')]
ax.legend(handles, ['GPDN-QHC', 'LBEG-QHC'], frameon = False,
          loc = 'upper right', bbox_to_anchor = (2, 1.15))


# (b) western part =====
ax = fig.add_axes([0.7, 0, 0.3, 1])
for k, (bid, dist) in enumerate(central_id.items()):
    profile = boreprofiles.loc[boreprofiles.ID.values == bid]
    profile = profile.reset_index(drop = True)
    
    profile = pd.concat([pd.DataFrame(data = dict(profile.loc[0]), index = [0]), profile])
    profile = profile.reset_index(drop = True)
    profile.Z[0] += profile.Depth[0]
    profile = profile.assign(thickness = np.abs(profile.Z.diff()))
    
    profile_color = [np.where(layer == bore_def.UnitID.values)[0][0]\
                      for layer in profile.UnitID.values]
    
    for idx, layer in profile.iterrows():
        b = ax.bar(dist, bottom = layer.Z, height = layer.thickness, width = 20,
                   facecolor = cmap_bores(profile_color[idx]), edgecolor = '#e6e5fe')
        if layer.UnitID == 'UNC_qhSTU':
            b.patches[0].set_hatch('//')
        if layer.UnitID == 'UNC_qhSTU_lag':
            b.patches[0].set_hatch('-')
        ax.bar(dist, bottom = layer.Z, height = layer.thickness, width = 20,
                   facecolor = 'none', edgecolor = 'k', zorder = 10)

ax.annotate('', 
            xy = (central_id['2209HY0040'], 8),
            xytext = (central_id['NyB19'], 8),
            arrowprops = {'arrowstyle': '<->'},
            annotation_clip = False)
ax.text(0.5 * (central_id['NyB19'] - central_id['2209HY0040']) + central_id['2209HY0040'],
        9, f"{central_id['NyB19'] - central_id['2209HY0040']} m",
        horizontalalignment = 'center')

ax.annotate('', 
            xy = (central_id['NyB19'], 8),
            xytext = (central_id['2209HY0017'], 8),
            arrowprops = {'arrowstyle': '<->'},
            annotation_clip = False)
ax.text(0.5 * (central_id['2209HY0017'] - central_id['NyB19']) + central_id['NyB19'],
        9, f"{central_id['2209HY0017'] - central_id['NyB19']} m",
        horizontalalignment = 'center')

ax.text(0.03, 0.96, '(b)', transform = ax.transAxes)

ax.set_ylim(-50, 12)

ax.text(ax.get_xlim()[0], ax.get_ylim()[1] + 1, '$\mathbf{N}$')
ax.text(ax.get_xlim()[1], ax.get_ylim()[1] + 1, '$\mathbf{S}$', 
        horizontalalignment = 'right')


ax.xaxis.tick_top()
ax.set_xticks(list(central_id.values()))
ax.set_xticklabels(central_id.keys(), rotation = 45, horizontalalignment = 'left')

ax.set_ylabel('Elevation [m NHN]')


# Model area ====
axs = fig.add_axes([0.33, 0.7, 0.3, 0.4])
axs.axis('off')

# Norderney area
xcoast, ycoast = coast['geometry'][0].xy
axs.plot(xcoast, ycoast, 'k', zorder = 0, linewidth = 0.5)

xmhw, ymhw = mhw['geometry'][0].xy
axs.plot(xmhw, ymhw, 'k--', zorder = 0, linewidth = 0.5)

for k, (bid, dist) in enumerate(west_id.items()):
    profile = boreprofiles.loc[boreprofiles.ID.values == bid]
    
    axs.scatter(profile.X, profile.Y, c = 'k', s = 10)
axs.text(0.04, 0.72, '(a)', transform = axs.transAxes)
    
for k, (bid, dist) in enumerate(central_id.items()):
    profile = boreprofiles.loc[boreprofiles.ID.values == bid]
    
    axs.scatter(profile.X, profile.Y, c = 'k', s = 10)
axs.text(0.4, 0.5, '(b)', transform = axs.transAxes)

axs.set_aspect(1)


# Legend =====
ids_leg = ['qh/S', 'qh/T-U', 'qh/H', 'qp/S', 'tpl/S', 'UNC_qhqp',
           'UNC_qhSTU', 'UNC_qhSTU_lag', 'UNC']
is_leg = [val in ids_leg for val in bore_def.UnitID.values]
bore_leg = bore_def.iloc[is_leg]

handles = []
labels = []
for (idx, row) in bore_leg.iterrows():
    if row.UnitID == 'UNC_qhqp':
        handles.append(
            (patches.Patch(edgecolor = '#bdf2da', facecolor = row.color,
                          hatch = '//'),
             patches.Patch(edgecolor = 'k', facecolor = 'none'))
        )
        labels.append('qh/S or qp/S')
    elif row.UnitID == 'UNC_qhSTU':
        handles.append(
            (patches.Patch(edgecolor = '#e6e5fe', facecolor = row.color,
                          hatch = '//'),
             patches.Patch(edgecolor = 'k', facecolor = 'none'))
        )
        labels.append('qh/S with large minor\nfraction C-U')
    elif row.UnitID == 'UNC_qhSTU_lag':
        handles.append(
            (patches.Patch(edgecolor = '#e6e5fe', facecolor = row.color,
                          hatch = '-'),
             patches.Patch(edgecolor = 'k', facecolor = 'none'))
        )
        labels.append('qh/S with layers of C-U')
    else:
        handles.append(
            patches.Patch(edgecolor = 'k', facecolor = row.color)    
        )
        if row.UnitID == 'qh/T-U':
            labels.append('qh/C-U')
        else:
            labels.append(row.UnitID)
    
ax.legend(handles = handles, labels = labels, frameon = False,
          loc = 'lower left', bbox_to_anchor = (-1.25, 0),
          handleheight = 2.6, handlelength = 2.5)

fig.savefig('Fig3.png', bbox_inches = 'tight', dpi = fig.dpi)
