#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Sep 30 13:20:46 2022

@author: patrick
"""

import os
import sys
import numpy as np
import netCDF4 as ncdf
import pandas as pd
import geopandas as gpd
import matplotlib.pyplot as plt
from matplotlib import rc
import matplotlib.colors as colors
import matplotlib.patches as patches
from matplotlib import rc

rc('font',**{'family':'serif',
             'size' : 10})
rc('text', usetex=True)
rc('axes', titlesize = 10, labelsize = 10)


fontsize = 10
font = {'family' : 'normal',
        'weight' : 'normal',
        'size'   : fontsize}

rc('font', **font)

# Set working directory to path of the script
os.chdir(os.path.dirname(os.path.realpath(__file__)))

path_wf = os.path.join('..', 'workflow')
path_bin = os.path.abspath(os.path.join(path_wf, 'C_hgsm_nor_3d', 'bin'))
sys.path.append(path_bin)

import raster_support as rs


#%% Load data
path_proj = os.path.join('..', 'workflow', 'C_hgsm_nor_3d')

# Topography for plotting
path_topo_plt = os.path.join(path_proj, 'raster', 'topography_hgsm_nor.tif')
topo_rs_plt = rs.Raster(path_topo_plt, get_array = True)
nodata = topo_rs_plt.raster.profile['nodata']
topo_plt = topo_rs_plt.array[0]
topo_plt[topo_plt == nodata] = np.nan

# Spatial information from raster data
nx = topo_rs_plt.raster.profile['width']
ny = topo_rs_plt.raster.profile['height']

dx = topo_rs_plt.raster.profile['transform'][0]
dy = topo_rs_plt.raster.profile['transform'][4] # is negative

xmin = topo_rs_plt.raster.profile['transform'][2]
ymax = topo_rs_plt.raster.profile['transform'][5]

x = xmin + np.arange(0, nx) * dx + 0.5 * dx
y = ymax + np.arange(0, ny) * dy + 0.5 * dy

X, Y = np.meshgrid(x, y)

xmax = xmin + nx*dx
ymin = ymax + ny*dy


# Topography
path_topo = os.path.join('..', 'hydrogeological_model_layer_raster', 
                         'topography_hgsm_nor_resampled_dx20m_dy20m.tif')
topo_rs = rs.Raster(path_topo, get_array = True)
nodata = topo_rs.raster.profile['nodata']
topo = topo_rs.array[0]
topo[topo == nodata] = np.nan

path_model = os.path.join('..', 'hydrogeological_model_3D_voxel')

# Model extent
path_extent = os.path.join(path_model, 'extent.csv')
extent = pd.read_csv(path_extent)

zmin = extent.zmin.min()
zmax = extent.zmax.max()

# Read data from netCDF file
path_ncdf = os.path.join(path_model, 'hgsm_nor.nc')
spatial_data = ncdf.Dataset(path_ncdf, 'r')

# Grid data
x = spatial_data['regular_grid']['easting'][:].data
y = spatial_data['regular_grid']['northing'][:].data
z = spatial_data['regular_grid']['elevation'][:].data
mask_topo = spatial_data['regular_grid']['mask_topo'][:].data
mask_topo = mask_topo.astype(bool)

dx = np.unique(np.diff(x))[0]
dy = np.unique(np.diff(y))[0]
dz = np.abs(np.diff(z))

# Unit IDs
surface = np.round(spatial_data['solutions']['lith_block'][:].data)
idvals = np.unique(surface)

nz, ny, nx = surface.shape # layer, row, column

# Get model resolution
nlay, nrow, ncol = surface.shape

spatial_data.close()


#%% 

# A - A', East-West ===========================================================
cs_name = 'EW-1'

# Cross-sections
path_cs = os.path.join(path_proj, 'shapes', 'cross_sections_nor.shp')
cs_nor = gpd.read_file(path_cs)

id_line = cs_nor['id'][cs_nor.CS_NAME == cs_name].values
cs_geom = cs_nor['geometry'][cs_nor.CS_NAME == cs_name][id_line[0]]
x_cs, y_cs = cs_geom.xy
x_cs = np.array(x_cs.tolist())
y_cs = np.array(y_cs.tolist())

# Cross-section point shape
# Created in QGIS using SAGA Profiles from Lines to extract a point for each
# underlying raster cell along a line shape. The raster was the resampled
# topography file with 25 x 25 m.
path_csp = os.path.join(path_proj, 'shapes', 'cross_sections_nor_points_raster.shp')
csp_nor = gpd.read_file(path_csp)

csp_geom = csp_nor['geometry'][csp_nor.LINE_ID.values == id_line]
x_csp, y_csp = [], []
for geom in csp_geom:
    x_csp.append(geom.xy[0][0])
    y_csp.append(geom.xy[1][0])
x_csp = np.array(x_csp)
y_csp = np.array(y_csp)
n_cells = x_csp.size

# Extract elevation
cs_topo = csp_nor.Z.values[csp_nor.LINE_ID.values == id_line]

# Get distance from cross-section origin for each point along cross-section
dist_points = ((x_csp[1:] - x_csp[:-1])**2 + (y_csp[1:] - y_csp[:-1])**2) ** 0.5
dist = np.array([0] + np.cumsum(dist_points).tolist())


# Series definitions
path_seriesdef = os.path.join(path_proj, 'input_data', 'cunitID_series_definition.csv')
series_def = pd.read_csv(path_seriesdef)

# Contains UnitID names and corresponding colors.
path_boredef = os.path.join(path_proj, 'input_data', 'unitID_definition.csv')
bore_def = pd.read_csv(path_boredef)

cmap = colors.ListedColormap(series_def.color.values)
cmap_bores = colors.ListedColormap(bore_def.color.values)

# Interpreted bore profile data
path_boreprofiles = os.path.join(path_proj, 'input_data', 'boredata_profiles_UnitID.csv')
boreprofiles = pd.read_csv(path_boreprofiles)

# Define and extract bores along cs
bid_use = ['9999GE1437', '2208IG0083', '2208IG0085', '2208IG0091',
           '2208HY0121', '2208HY0052', '2208IG0070', '2208HY0055',
           '2208HY0056', '2208HY0054', '2208HY0058', '2208HY0059',
           'NyB40E', '2208HY0098', '2209HY0144', 'NyB28E',
           #'NyB37E', 
           'NyB31E', 'NyB42E', 'NyB34E', '2209IG0067', 'SN6', 'SN7',
           '2209HY0074', '2209HY0075', 'NyBS11', '2209HY0132', '2209HY0015',
           '2209HY0014', '2209HY0001', 'NyB19', '2209HY0017', 'NyB18',
           '2209HY0018', '2209HY0019', '2209HY0046',
           '2209HY0020', '2209HY0051', '2209HY0135', '2209HY0128',
           '2209IG0077', '2209IG0078', '2210GE0002', '9999IG2904',
           '2210GE0046']

bores = pd.DataFrame(columns = boreprofiles.columns)
for bid in bid_use:
    bores = bores.append(boreprofiles.iloc[bid == boreprofiles.ID.values])

ref = bores.drop_duplicates(subset = 'ID')
distb = []
for k, thisbore in ref.iterrows():
    # Get minimum distance of bore to cs to map bore on cross-section
    dist_bore = ((thisbore.X - x_csp) ** 2 + (thisbore.Y - y_csp) ** 2) ** 0.5
    idx_mindistb = np.nanargmin(dist_bore)
    distb.append(dist[idx_mindistb])
ref.insert(1, "DIST", distb)


# Find cells that are crossed by section
# The row and cols can be found using the topo array spatial info.
idx_x, idx_y = [], []
for (xpos, ypos) in zip(x_csp, y_csp):
    idx_x.append(np.nanargmin(np.abs(xpos - x)))
    idx_y.append(np.nanargmin(np.abs(ypos - y)))
idx_x = np.array(idx_x)
idx_y = np.array(idx_y)

# Extract surface info
cs_lith = surface[::, idx_y, idx_x]

# Make a grid for plotting
DIST, Z = np.meshgrid(dist, z)

# Set values above topography to nan
CS_TOPO = np.repeat(cs_topo[np.newaxis, ::], Z.shape[0], axis = 0)
above_topo = Z > CS_TOPO

cs_lith[above_topo] = np.nan

zlim = [-50, 25]

dlim = [0, 9000]
idx_part = (dist > dlim[0]) & (dist < dlim[1])

fig = plt.figure(0, figsize = (17.6/2.54, 22/2.54))
plt.tight_layout()

ax = fig.add_axes([0, 0, 1, 1], frameon=False)
ax.tick_params(labelcolor='none', top=False, bottom=False, left=False, right=False)
ax.grid(False)
ax.set_ylabel('Elevation [m NHN]', labelpad = 10)

# Cross-section
axa = fig.add_axes([0, 0.7875, 1, 0.2125])

# Topography
axa.step(dist, cs_topo, 'k', where = 'mid', linewidth = 0.5)

# CUnits
im = axa.pcolormesh(DIST[:800], Z[:800], cs_lith[:800], shading = 'nearest', cmap = cmap, zorder = 1)

axa.set_xlim(dlim[0], dlim[1])
axa.set_ylim(zlim)

axa.set_xticks(np.arange(dlim[0], dlim[1]+1, step = 1000))
axa.set_xticklabels(axa.get_xticks() - dlim[0])

yticks = np.arange(zlim[0], zlim[1]+1, step = 10)
axa.set_yticks(yticks)

#axa.set_xlabel('Distance to origin [m]')
#axa.set_ylabel('Elevation [m NHN]')

#axa.set_aspect(25)

axa.text(dlim[0], zlim[1]+0.1*zlim[1], '$\\mathbf{A}$',
        horizontalalignment = 'center')
axa.text(dlim[1], zlim[1]+0.1*zlim[1], "$\\mathbf{A'}$",
        horizontalalignment = 'center')


# Add bore data -----
# Add names of bores
# axa.vlines(ref.DIST.values, ymin = zmin, 
#            ymax = 10, #ymax = ref.Z.values + ref.Depth.values + 10, 
#            color = 'k', alpha = 1, 
#            linewidth = 0.5, zorder = 0)
# for k, (ind, layer) in enumerate(ref.iterrows()):
#     if (layer.DIST > axa.get_xlim()[0]) & (layer.DIST < axa.get_xlim()[1]):
#         xpos = layer.DIST - 20 if not layer.ID == 'NyB42E' else layer.DIST
#         axa.text(xpos, 11, layer.ID, rotation = 30, 
#                  horizontalalignment = 'left',
#                  fontsize = fontsize-4)
   
for ind, bore_sel in ref.iterrows():
    profile = bores.iloc[bores.ID.values == bore_sel.ID]
    profile_color = [np.where(layer == bore_def.UnitID.values)[0][0]\
                      for layer in profile.UnitID.values]
        
    # Add profiles
    addval = 20
    
    surface_elevation = profile.Z.values[-1] + profile.Depth.values[-1]
    top = np.array([surface_elevation] + profile.Z.values.tolist())[:-1]
    bottom = profile.Z.values
    
    xprofile = bore_sel.DIST
    
    for k in range(profile.index.size):
        poly = patches.Polygon([[xprofile - addval, top[k]],
                                [xprofile + addval, top[k]],
                                [xprofile + addval, bottom[k]],
                                [xprofile - addval, bottom[k]]],
                                facecolor = cmap_bores(profile_color[k]),
                                edgecolor = 'k', alpha = 1,
                                linewidth = 0.1, zorder = 20)
    
        axa.add_patch(poly)    
 
# Ocean
poly = patches.Polygon([[dlim[0], zlim[0]], 
                        [dlim[0], 0],
                        [dlim[1], 0], 
                        [dlim[1], zlim[0]]],
                       facecolor = '#a0c3cc',
                       edgecolor = 'none', zorder = 0)
axa.add_patch(poly)


# A' - A'', East-West ===========================================================
cs_name = 'EW-1'

# Cross-sections
path_cs = os.path.join(path_proj, 'shapes', 'cross_sections_nor.shp')
cs_nor = gpd.read_file(path_cs)

id_line = cs_nor['id'][cs_nor.CS_NAME == cs_name].values
cs_geom = cs_nor['geometry'][cs_nor.CS_NAME == cs_name][id_line[0]]
x_cs, y_cs = cs_geom.xy
x_cs = np.array(x_cs.tolist())
y_cs = np.array(y_cs.tolist())

# Cross-section point shape
# Created in QGIS using SAGA Profiles from Lines to extract a point for each
# underlying raster cell along a line shape. The raster was the resampled
# topography file with 25 x 25 m.
path_csp = os.path.join(path_proj, 'shapes', 'cross_sections_nor_points_raster.shp')
csp_nor = gpd.read_file(path_csp)

csp_geom = csp_nor['geometry'][csp_nor.LINE_ID.values == id_line]
x_csp, y_csp = [], []
for geom in csp_geom:
    x_csp.append(geom.xy[0][0])
    y_csp.append(geom.xy[1][0])
x_csp = np.array(x_csp)
y_csp = np.array(y_csp)
n_cells = x_csp.size

# Extract elevation
cs_topo = csp_nor.Z.values[csp_nor.LINE_ID.values == id_line]

# Get distance from cross-section origin for each point along cross-section
dist_points = ((x_csp[1:] - x_csp[:-1])**2 + (y_csp[1:] - y_csp[:-1])**2) ** 0.5
dist = np.array([0] + np.cumsum(dist_points).tolist())


# Series definitions
path_seriesdef = os.path.join(path_proj, 'input_data', 'cunitID_series_definition.csv')
series_def = pd.read_csv(path_seriesdef)

# Contains UnitID names and corresponding colors.
path_boredef = os.path.join(path_proj, 'input_data', 'unitID_definition.csv')
bore_def = pd.read_csv(path_boredef)

cmap = colors.ListedColormap(series_def.color.values)
cmap_bores = colors.ListedColormap(bore_def.color.values)

# Interpreted bore profile data
path_boreprofiles = os.path.join(path_proj, 'input_data', 'boredata_profiles_UnitID.csv')
boreprofiles = pd.read_csv(path_boreprofiles)

# Define and extract bores along cs
bid_use = ['9999GE1437', '2208IG0083', '2208IG0085', '2208IG0091',
           '2208HY0121', '2208HY0052', '2208IG0070', '2208HY0055',
           '2208HY0056', '2208HY0054', '2208HY0058', '2208HY0059',
           'NyB40E', '2208HY0098', '2209HY0144', 'NyB28E',
           #'NyB37E', 
           'NyB31E', 'NyB42E', 'NyB34E', '2209IG0067', 'SN6', 'SN7',
           '2209HY0074', '2209HY0075', 'NyBS11', '2209HY0132', '2209HY0015',
           '2209HY0014', '2209HY0001', 'NyB19', '2209HY0017', 'NyB18',
           '2209HY0018', '2209HY0019', '2209HY0046',
           '2209HY0020', '2209HY0051', '2209HY0135', '2209HY0128',
           '2209IG0077', '2209IG0078', '2210GE0002', '9999IG2904',
           '2210GE0046']

bores = pd.DataFrame(columns = boreprofiles.columns)
for bid in bid_use:
    bores = bores.append(boreprofiles.iloc[bid == boreprofiles.ID.values])

ref = bores.drop_duplicates(subset = 'ID')
distb = []
for k, thisbore in ref.iterrows():
    # Get minimum distance of bore to cs to map bore on cross-section
    dist_bore = ((thisbore.X - x_csp) ** 2 + (thisbore.Y - y_csp) ** 2) ** 0.5
    idx_mindistb = np.nanargmin(dist_bore)
    distb.append(dist[idx_mindistb])
ref.insert(1, "DIST", distb)


# Find cells that are crossed by section
# The row and cols can be found using the topo array spatial info.
idx_x, idx_y = [], []
for (xpos, ypos) in zip(x_csp, y_csp):
    idx_x.append(np.nanargmin(np.abs(xpos - x)))
    idx_y.append(np.nanargmin(np.abs(ypos - y)))
idx_x = np.array(idx_x)
idx_y = np.array(idx_y)

# Extract surface info
cs_lith = surface[::, idx_y, idx_x]

# Make a grid for plotting
DIST, Z = np.meshgrid(dist, z)

# Set values above topography to nan
CS_TOPO = np.repeat(cs_topo[np.newaxis, ::], Z.shape[0], axis = 0)
above_topo = Z > CS_TOPO

cs_lith[above_topo] = np.nan

zlim = [-50, 25]

dlim = [9000, dist.max()]
idx_part = (dist > dlim[0]) & (dist < dlim[1])

fig = plt.figure(0, figsize = (17.6/2.54, 22/2.54))
plt.tight_layout()

ax = fig.add_axes([0, 0, 1, 1], frameon=False)
ax.tick_params(labelcolor='none', top=False, bottom=False, left=False, right=False)
ax.grid(False)
ax.set_ylabel('Elevation [m NHN]', labelpad = 10)

# Cross-section
axb = fig.add_axes([0, 0.5, 1, 0.2125])

# Topography
axb.step(dist, cs_topo, 'k', where = 'mid', linewidth = 0.5)

# CUnits
im = axb.pcolormesh(DIST[:800], Z[:800], cs_lith[:800], shading = 'nearest', cmap = cmap, zorder = 1)

axb.set_xlim(dlim[0], dlim[1])
axb.set_ylim(zlim)

axb.set_xticks(np.arange(dlim[0], dlim[1]+1, step = 1000))
axb.set_xticklabels(axb.get_xticks().astype(int))

yticks = np.arange(zlim[0], zlim[1]+1, step = 10)
axb.set_yticks(yticks)

#axb.set_xlabel('Distance to origin [m]')
#axb.set_ylabel('Elevation [m NHN]')

#axb.set_aspect(25)

axb.text(dlim[0], zlim[1]+0.1*zlim[1], "$\\mathbf{A'}$",
        horizontalalignment = 'center')
axb.text(dlim[1], zlim[1]+0.1*zlim[1], "$\\mathbf{A''}$",
        horizontalalignment = 'center')


# Add bore data -----
# Add names of bores
# axb.vlines(ref.DIST.values, ymin = zmin, 
#            ymax = 10, #ymax = ref.Z.values + ref.Depth.values + 10, 
#            color = 'k', alpha = 1, 
#            linewidth = 0.5, zorder = 0)
# for k, (ind, layer) in enumerate(ref.iterrows()):
#     if (layer.DIST > axb.get_xlim()[0]) & (layer.DIST < axb.get_xlim()[1]):
#         xpos = layer.DIST - 20 if not layer.ID == 'NyB42E' else layer.DIST
#         axb.text(xpos, 11, layer.ID, rotation = 30, 
#                  horizontalalignment = 'left',
#                  fontsize = fontsize-4)
   
for ind, bore_sel in ref.iterrows():
    profile = bores.iloc[bores.ID.values == bore_sel.ID]
    profile_color = [np.where(layer == bore_def.UnitID.values)[0][0]\
                      for layer in profile.UnitID.values]
        
    # Add profiles
    addval = 20
    
    surface_elevation = profile.Z.values[-1] + profile.Depth.values[-1]
    top = np.array([surface_elevation] + profile.Z.values.tolist())[:-1]
    bottom = profile.Z.values
    
    xprofile = bore_sel.DIST
    
    for k in range(profile.index.size):
        poly = patches.Polygon([[xprofile - addval, top[k]],
                                [xprofile + addval, top[k]],
                                [xprofile + addval, bottom[k]],
                                [xprofile - addval, bottom[k]]],
                                facecolor = cmap_bores(profile_color[k]),
                                edgecolor = 'k', alpha = 1,
                                linewidth = 0.1, zorder = 20)
    
        axb.add_patch(poly)    
 
# Ocean
poly = patches.Polygon([[dlim[0], zlim[0]], 
                        [dlim[0], 0],
                        [dlim[1], 0], 
                        [dlim[1], zlim[0]]],
                       facecolor = '#a0c3cc',
                       edgecolor = 'none', zorder = 0)
axb.add_patch(poly)


# B - B', North-East in West ==================================================
cs_name = 'NS-1'

# Cross-sections
path_cs = os.path.join(path_proj, 'shapes', 'cross_sections_nor.shp')
cs_nor = gpd.read_file(path_cs)

id_line = cs_nor['id'][cs_nor.CS_NAME == cs_name].values
cs_geom = cs_nor['geometry'][cs_nor.CS_NAME == cs_name][id_line[0]]
x_cs, y_cs = cs_geom.xy
x_cs = np.array(x_cs.tolist())
y_cs = np.array(y_cs.tolist())
# Cross-section point shape
# Created in QGIS using SAGA Profiles from Lines to extract a point for each
# underlying raster cell along a line shape. The raster was the resampled
# topography file with 25 x 25 m.
path_csp = os.path.join(path_proj, 'shapes', 'cross_sections_nor_points_raster.shp')
csp_nor = gpd.read_file(path_csp)

csp_geom = csp_nor['geometry'][csp_nor.LINE_ID.values == id_line]
x_csp, y_csp = [], []
for geom in csp_geom:
    x_csp.append(geom.xy[0][0])
    y_csp.append(geom.xy[1][0])
x_csp = np.array(x_csp)
y_csp = np.array(y_csp)
n_cells = x_csp.size

# Extract elevation
cs_topo = csp_nor.Z.values[csp_nor.LINE_ID.values == id_line]

# Get distance from cross-section origin for each point along cross-section
dist_points = ((x_csp[1:] - x_csp[:-1])**2 + (y_csp[1:] - y_csp[:-1])**2) ** 0.5
dist = np.array([0] + np.cumsum(dist_points).tolist())


# Series definitions
path_seriesdef = os.path.join(path_proj, 'input_data', 'cunitID_series_definition.csv')
series_def = pd.read_csv(path_seriesdef)

# Contains UnitID names and corresponding colors.
path_boredef = os.path.join(path_proj, 'input_data', 'unitID_definition.csv')
bore_def = pd.read_csv(path_boredef)

cmap = colors.ListedColormap(series_def.color.values)
cmap_bores = colors.ListedColormap(bore_def.color.values)

# Interpreted bore profile data
path_boreprofiles = os.path.join(path_proj, 'input_data', 'boredata_profiles_UnitID.csv')
boreprofiles = pd.read_csv(path_boreprofiles)

# Define and extract bores along cs
# Define and extract bores along cs
bid_use = ['SN3', '2208IG0069', '2208IG0046', '2208HY0055',
           '2208HY0060', '2208IG0009', '2208HY0003', 'NyBS7', '2208IG0045',
           '2208IG0040', '2208IG0064', '2208IG0106', '2308IG0014',
           '2309SE0001', '2309SE0004', '2309SE0002', '2309SE0007',
           '2309GE0080', '2309GE0014']

bores = pd.DataFrame(columns = boreprofiles.columns)
for bid in bid_use:
    bores = bores.append(boreprofiles.iloc[bid == boreprofiles.ID.values])

ref = bores.drop_duplicates(subset = 'ID')
distb = []
for k, thisbore in ref.iterrows():
    # Get minimum distance of bore to cs to map bore on cross-section
    dist_bore = ((thisbore.X - x_csp) ** 2 + (thisbore.Y - y_csp) ** 2) ** 0.5
    idx_mindistb = np.nanargmin(dist_bore)
    distb.append(dist[idx_mindistb])
ref.insert(1, "DIST", distb)


# Find cells that are crossed by section
# The row and cols can be found using the topo array spatial info.
idx_x, idx_y = [], []
for (xpos, ypos) in zip(x_csp, y_csp):
    idx_x.append(np.nanargmin(np.abs(xpos - x)))
    idx_y.append(np.nanargmin(np.abs(ypos - y)))
idx_x = np.array(idx_x)
idx_y = np.array(idx_y)

# Extract surface info
cs_lith = surface[::, idx_y, idx_x]

# Make a grid for plotting
DIST, Z = np.meshgrid(dist, z)

# Set values above topography to nan
CS_TOPO = np.repeat(cs_topo[np.newaxis, ::], Z.shape[0], axis = 0)
above_topo = Z > CS_TOPO

cs_lith[above_topo] = np.nan

zlim = [-50, 25]

dlim = [0, dist.max()]
idx_part = (dist > dlim[0]) & (dist < dlim[1])
   
axc = fig.add_axes([0, 0.2125, 1, 0.2125])

# Topography
axc.step(dist, cs_topo, 'k', where = 'mid', linewidth = 1)

# CUnits
im = axc.pcolormesh(DIST, Z, cs_lith, shading = 'nearest', cmap = cmap, zorder = 1)


axc.set_xlim(dlim[0], dlim[1])
axc.set_ylim(zlim)

axc.set_xticks(np.arange(dlim[0], dlim[1]+1, step = 500))

yticks = np.arange(zlim[0], zlim[1]+1, step = 10)
axc.set_yticks(yticks)

axc.set_xlabel('Distance to cross-section origin [m]')
#axc.set_ylabel('Elevation [m NHN]')

#axc.set_aspect(12.5)

axc.text(dlim[0], zlim[1]+0.1*zlim[1], '$\\mathbf{B}$',
        horizontalalignment = 'center', fontsize = fontsize)
axc.text(dlim[1], zlim[1]+0.1*zlim[1], "$\\mathbf{B'}$",
        horizontalalignment = 'center')


# Add bore data -----
# Add names of bores
# axc.vlines(ref.DIST.values, ymin = zmin, 
#            ymax = 10, color = 'k', alpha = 1, 
#           linewidth = 0.5, zorder = 0)
# for k, (ind, layer) in enumerate(ref.iterrows()):
#     if (layer.DIST > axc.get_xlim()[0]) & (layer.DIST < axc.get_xlim()[1]):
#         xpos = layer.DIST - 20
#         if (layer.ID == 'NyB42E'):
#             xpos = layer.DIST - 30
#         if (layer.ID == 'NyB37E'):
#             xpos = layer.DIST
#         if (layer.ID == 'NyB36E'):
#             xpos = layer.DIST + 15
#         axc.text(xpos, 11, layer.ID, rotation = 30, horizontalalignment = 'left',
#                 fontsize = fontsize-4)
   
for ind, bore_sel in ref.iterrows():
    profile = bores.iloc[bores.ID.values == bore_sel.ID]
    profile_color = [np.where(layer == bore_def.UnitID.values)[0][0]\
                      for layer in profile.UnitID.values]
        
    # Add profiles
    addval = 10
    
    surface_elevation = profile.Z.values[-1] + profile.Depth.values[-1]
    top = np.array([surface_elevation] + profile.Z.values.tolist())[:-1]
    bottom = profile.Z.values
    
    xprofile = bore_sel.DIST
    
    for k in range(profile.index.size):
        poly = patches.Polygon([[xprofile - addval, top[k]],
                                [xprofile + addval, top[k]],
                                [xprofile + addval, bottom[k]],
                                [xprofile - addval, bottom[k]]],
                                facecolor = cmap_bores(profile_color[k]),
                                edgecolor = 'k', alpha = 1,
                                linewidth = 0.1, zorder = 20)
    
        axc.add_patch(poly)    
            
        
# Ocean
poly = patches.Polygon([[dlim[0], zlim[0]], 
                        [dlim[0], 0],
                        [dlim[1], 0], 
                        [dlim[1], zlim[0]]],
                       facecolor = '#a0c3cc',
                       edgecolor = 'none', zorder = 0)
axc.add_patch(poly)


# Legends -----
# Create legend UnitIDs
handles_uid = [patches.Patch(facecolor = uid_color, edgecolor = 'black')\
                for uid_color in bore_def.color.values]
labels_uid = bore_def.UnitID.values

leg_uid = ax.legend(handles_uid, labels_uid, title = '$\\bf{Bore\ classification\ units}$',
                    loc = 'lower left', ncol = 7, frameon = False,
                    bbox_to_anchor = (-0.07, -0.03))
leg_uid._legend_box.align = 'left'


# Create legend CUnitIDs
series_def_sub = series_def.iloc[series_def.surface.values != 'qp/S_2']
handles_cuid = [patches.Patch(facecolor = cuid_color, edgecolor = 'black')\
                for cuid_color in series_def_sub.color.values[0:5]]
labels_cuid = series_def_sub.surface.values[0:5]

leg_cuid = ax.legend(handles_cuid, labels_cuid, title = '$\\bf{Conceptional\ model\ units}$',
                     loc = 'lower left', frameon = False, ncol = 5,
                     bbox_to_anchor = (-0.07, 0.08),
                     columnspacing = leg_uid.columnspacing + 0.1)
leg_cuid._legend_box.align = 'left'

ax.add_artist(leg_uid)

fig.savefig('Fig4.png', bbox_inches = 'tight', dpi = fig.dpi)
