#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Sep 30 16:11:19 2022

@author: patrick
"""

import os
import sys
import numpy as np
import geopandas as gpd
import descartes as dc
import matplotlib.pyplot as plt
import matplotlib.cm as cm
from matplotlib import lines
from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
from matplotlib import rc

rc('font',**{'family':'serif',
             'size' : 10})
rc('text', usetex=True)
rc('axes', titlesize = 10, labelsize = 10)

# Set working directory to path of the script
os.chdir(os.path.dirname(os.path.realpath(__file__)))

path_proj = os.path.join('..', 'workflow')
path_bin = os.path.abspath(os.path.join(path_proj, 'C_hgsm_nor_3d', 'bin'))
sys.path.append(path_bin)

import raster_support as rs

#%% Load data 

# Norderney coastline
path_0m = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                       'shapes', 'nor_coastline_0masl.shp')
coast = gpd.read_file(path_0m)

path_mhw = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                        'shapes', 'nor_mhw-line_125cmasl.shp')
mhw = gpd.read_file(path_mhw)

# Model area HGSM 3D
path_hgsm = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                         'shapes', 'modelarea_hgsm_norderney_epsg25832.shp')
hgsm_area = gpd.read_file(path_hgsm)


# qh/T-U
path_qhtu = os.path.join('..', 'hydrogeological_model_layer_raster',
                         'qh_T-U_base_hgsm_norderney.tif')
qhtu_rs = rs.Raster(path_qhtu, get_array = True)
nodata = qhtu_rs.raster.profile['nodata']
qhtu = qhtu_rs.array[0]
qhtu[qhtu == nodata] = np.nan

path_qhtu_thick = os.path.join('..', 'hydrogeological_model_layer_raster',
                         'qh_T-U_thickness_hgsm_norderney.tif')
qhtuthick_rs = rs.Raster(path_qhtu_thick, get_array = True)
nodata = qhtuthick_rs.raster.profile['nodata']
qhtuthick = qhtuthick_rs.array[0]
qhtuthick[qhtuthick == nodata] = np.nan

path_qhtubound = os.path.join(path_proj, 'C_hgsm_nor_3d', 'shape_layers',
                              'qh_T-U_base_hgsm_norderney_dissolved.shp')
qhtubound = gpd.read_file(path_qhtubound)

# qp/T-U
path_qptu = os.path.join('..', 'hydrogeological_model_layer_raster',
                         'qp_T-U_base_hgsm_norderney.tif')
qptu_rs = rs.Raster(path_qptu, get_array = True)
nodata = qptu_rs.raster.profile['nodata']
qptu = qptu_rs.array[0]
qptu[qptu == nodata] = np.nan

path_qptu_thick = os.path.join('..', 'hydrogeological_model_layer_raster',
                         'qp_T-U_thickness_hgsm_norderney.tif')
qptuthick_rs = rs.Raster(path_qptu_thick, get_array = True)
nodata = qptuthick_rs.raster.profile['nodata']
qptuthick = qptuthick_rs.array[0]
qptuthick[qptuthick == nodata] = np.nan

path_qptubound = os.path.join(path_proj, 'C_hgsm_nor_3d', 'shape_layers',
                              'qp_T-U_base_hgsm_norderney_dissolved.shp')
qptubound = gpd.read_file(path_qptubound)


x = np.array([qhtu_rs.raster.xy(0, k)[0] for k in range(qhtu_rs.raster.profile['width'])])
y = np.array([qhtu_rs.raster.xy(k, 0)[1] for k in range(qhtu_rs.raster.profile['height'])])

X, Y = np.meshgrid(x, y)

xmin, xmax = x.min(), x.max()
ymin, ymax = y.min(), y.max()


#%% Plot
lvl = np.arange(-23, -3, 1)

fig = plt.figure(0, figsize = (17.6/2.54, 16.5/2.54))

# qh/T-U =====
ax = fig.add_axes([0, 0.5, 1, 0.45])

cax = fig.add_axes([0.53, 0.07, 0.45, 0.025])
cmap_diff = cm.get_cmap('gray_r', 21)
im = ax.pcolormesh(X, Y, qhtuthick, cmap = cmap_diff, vmin = 0, vmax = 10.5,
                    zorder = 0)
cbar_diff = fig.colorbar(im, ax = ax, cax = cax, orientation = 'horizontal',
                          fraction = 0.1)
cbar_diff.set_label('Thickness [m]')

# Norderney area
xcoast, ycoast = coast['geometry'][0].xy
ax.plot(xcoast, ycoast, 'k', zorder = 0, linewidth = 1)

xmhw, ymhw = mhw['geometry'][0].xy
ax.plot(xmhw, ymhw, 'k--', zorder = 0, linewidth = 1)

# HGSM model area
hgsmpoly = dc.PolygonPatch(hgsm_area['geometry'][0], facecolor = 'none', edgecolor = 'k',
                           zorder = 0)
ax.add_patch(hgsmpoly)

cntr = ax.contour(X, Y, qhtu, levels = lvl, linestyles = '-',
                  cmap = 'viridis_r', linewidths = 1)

qhtu_patch = dc.PolygonPatch(qhtubound['geometry'][0], facecolor = 'none', 
                             edgecolor = 'k', linewidth = 0.8)
ax.add_patch(qhtu_patch)

ax.text(0.01, 0.94, '(a)', transform = ax.transAxes)

# North arrow
ax.arrow(xmax-5e2, ymax-9.5e2, 0, 0, head_width = 400, fc = 'black', ec = 'none',
         overhang = 0.2,
         zorder = 10)
ax.text(xmax-5e2, ymax-3.5e2, '$\\mathbf{N}$', ha = 'center')

# Scale bar
scalebar = AnchoredSizeBar(ax.transData, 1000, '1000 m', 'lower left',
                           frameon = False, size_vertical = 10, sep = 5,
                           bbox_to_anchor = [120, 1090])
scalebar.zorder = 200
ax.add_artist(scalebar)

ax.set_yticks(np.arange(y[-1].round(-3)+5e2, y[0].round(-3), 2e3).astype(int))
ax.set_yticklabels(ax.get_yticks(), rotation = 90, verticalalignment = 'center')

ax.set_xlim(xmin, xmax)
ax.set_ylim(ymin, ymax)

ax.set_aspect(1)

labels = lvl.astype(str)
handles = []
for k in range(len(labels)):
    handles.append(lines.Line2D([0], [0], color = cntr.collections[k].get_edgecolor()))
    
leg = ax.legend(handles, labels, loc = 'lower left',
          title = 'Base elevation [m NHN]',
          ncol = 4, frameon = False,
          bbox_to_anchor = (0.53, -0.875))
leg._legend_box.align = 'left'

ax.add_artist(leg)

handles_mhw = [lines.Line2D([0], [0], color = 'k', linestyle = '-'),
           lines.Line2D([0], [0], color = 'k', linestyle = '--')]
ax.legend(handles_mhw, ['0 m NHN', 'MHW = 1.25 m NHN'], 
          loc = 'lower left', frameon = False,
          bbox_to_anchor = (0.53, -0.375))


#qp/T-U =====
axb = fig.add_axes([0, 0, 0.5, 0.45])

im = axb.pcolormesh(X, Y, qptuthick, cmap = cmap_diff, vmin = 0, vmax = 10.5,
                    zorder = 0)

# Norderney area
xcoast, ycoast = coast['geometry'][0].xy
axb.plot(xcoast, ycoast, 'k', zorder = 0, linewidth = 1)

xmhw, ymhw = mhw['geometry'][0].xy
axb.plot(xmhw, ymhw, 'k--', zorder = 0, linewidth = 1)

# HGSM model area
hgsmpoly = dc.PolygonPatch(hgsm_area['geometry'][0], facecolor = 'none', edgecolor = 'k',
                           zorder = 0)
axb.add_patch(hgsmpoly)

cntr = axb.contour(X, Y, qptu, levels = lvl, linestyles = '-',
                  cmap = 'viridis_r', linewidths = 1)

qptu_patch = dc.PolygonPatch(qptubound['geometry'][0], facecolor = 'none', 
                             edgecolor = 'k', linewidth = 0.8)
axb.add_patch(qptu_patch)

axb.text(0.02, 0.94, '(b)', transform = axb.transAxes)

# North arrow
axb.arrow(384415-5e2, ymax-9.5e2, 0, 0, head_width = 400, fc = 'black', ec = 'none',
         overhang = 0.2,
         zorder = 10)
axb.text(384415-5e2, ymax-3.5e2, '$\\mathbf{N}$', ha = 'center')

# Scale bar
scalebar = AnchoredSizeBar(axb.transData, 1000, '1000 m', 'lower left',
                           frameon = False, size_vertical = 10, sep = 5,
                           bbox_to_anchor = [120, 120])
scalebar.zorder = 200
axb.add_artist(scalebar)

axb.set_yticks(np.arange(y[-1].round(-3)+5e2, y[0].round(-3), 2e3).astype(int))
axb.set_yticklabels(axb.get_yticks(), rotation = 90, verticalalignment = 'center')

axb.set_xlim(xmin, 384415)
axb.set_ylim(ymin, ymax)

axb.set_aspect(1)


fig.savefig('Fig5.png', bbox_inches = 'tight', dpi = fig.dpi)

