#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Sep 30 14:52:36 2022

@author: patrick
"""

import os
import sys
import numpy as np
import geopandas as gpd
import descartes as dc
import matplotlib.pyplot as plt
import matplotlib.cm as cm
from matplotlib import lines
from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
from matplotlib import rc

rc('font',**{'family':'serif',
             'size' : 10})
rc('text', usetex=True)
rc('axes', titlesize = 10, labelsize = 10)

# Set working directory to path of the script
os.chdir(os.path.dirname(os.path.realpath(__file__)))

path_proj = os.path.join('..', 'workflow')
path_bin = os.path.abspath(os.path.join(path_proj, 'C_hgsm_nor_3d', 'bin'))
sys.path.append(path_bin)

import raster_support as rs

#%% Load data 

# Norderney coastline
path_0m = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                       'shapes', 'nor_coastline_0masl.shp')
coast = gpd.read_file(path_0m)

path_mhw = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                        'shapes', 'nor_mhw-line_125cmasl.shp')
mhw = gpd.read_file(path_mhw)

# Model area HGSM 3D
path_hgsm = os.path.join(path_proj, 'C_hgsm_nor_3d', 
                         'shapes', 'modelarea_hgsm_norderney_epsg25832.shp')
hgsm_area = gpd.read_file(path_hgsm)


# tmim
path_tmim = os.path.join('..', 'hydrogeological_model_layer_raster',
                         'tmim_base_hgsm_norderney.tif')
tmim_rs = rs.Raster(path_tmim, get_array = True)
nodata = tmim_rs.raster.profile['nodata']
tmim = tmim_rs.array[0]
tmim[tmim == nodata] = np.nan

# teoo
path_teoo = os.path.join('..', 'hydrogeological_model_layer_raster',
                         'teoo_base_hgsm_norderney.tif')
teoo_rs = rs.Raster(path_teoo, get_array = True)
nodata = teoo_rs.raster.profile['nodata']
teoo = teoo_rs.array[0]
teoo[teoo == nodata] = np.nan

x = np.array([teoo_rs.raster.xy(0, k)[0] for k in range(teoo_rs.raster.profile['width'])])
y = np.array([teoo_rs.raster.xy(k, 0)[1] for k in range(teoo_rs.raster.profile['height'])])

X, Y = np.meshgrid(x, y)

xmin, xmax = x.min(), x.max()
ymin, ymax = y.min(), y.max()

#%% Plot
lvl = np.arange(-800, -375, 50)

fig = plt.figure(0, figsize = (17.6/2.54, 9/2.54))

ax = fig.add_axes([0, 0, 1, 0.8])

cax = fig.add_axes([0.53, -0.2, 0.45, 0.05])
cmap_diff = cm.get_cmap('gray_r', 22)
im = ax.pcolormesh(X, Y, tmim-teoo, cmap = cmap_diff, vmin = 0, vmax = 275,
                   zorder = 0)
cbar_diff = fig.colorbar(im, ax = ax, cax = cax, orientation = 'horizontal',
                         fraction = 0.1)
cbar_diff.set_label('Base elevation difference teoo to tmim [m]')

# Norderney area
xcoast, ycoast = coast['geometry'][0].xy
ax.plot(xcoast, ycoast, 'k', zorder = 0, linewidth = 1)

xmhw, ymhw = mhw['geometry'][0].xy
ax.plot(xmhw, ymhw, 'k--', zorder = 0, linewidth = 1)

# HGSM model area
hgsmpoly = dc.PolygonPatch(hgsm_area['geometry'][0], facecolor = 'none', edgecolor = 'k',
                           zorder = 0)
ax.add_patch(hgsmpoly)

ax.contour(X, Y, tmim, levels = lvl, linestyles = '--',
           cmap = 'viridis_r')
cntr = ax.contour(X, Y, teoo, levels = lvl, linestyles = '-',
                  cmap = 'viridis_r')

# North arrow
ax.arrow(xmax-5e2, ymax-9.5e2, 0, 0, head_width = 400, fc = 'black', ec = 'none',
         overhang = 0.2,
         zorder = 10)
ax.text(xmax-5e2, ymax-3.5e2, '$\\mathbf{N}$', ha = 'center')

# Scale bar
scalebar = AnchoredSizeBar(ax.transData, 1000, '1000 m', 'lower left',
                           frameon = False, size_vertical = 10, sep = 5,
                           bbox_to_anchor = [120, 390])
scalebar.zorder = 200
ax.add_artist(scalebar)

ax.set_yticks(np.arange(y[-1].round(-3)+5e2, y[0].round(-3), 2e3).astype(int))
ax.set_yticklabels(ax.get_yticks(), rotation = 90, verticalalignment = 'center')

ax.set_xlim(xmin, xmax)
ax.set_ylim(ymin, ymax)

ax.set_aspect(1)

labels = lvl.astype(str)
handles = []
for k in range(len(labels)):
    handles.append(lines.Line2D([0], [0], color = cntr.collections[k].get_edgecolor()))
    
leg = ax.legend(handles, labels, loc = 'lower left',
          title = 'Base elevation [m NHN]',
          ncol = 3, frameon = False,
          bbox_to_anchor = (-0.02, -0.435))
leg._legend_box.align = 'left'

ax.add_artist(leg)


handles_strat = [lines.Line2D([0], [0], color = handles[4].get_color(), linestyle = '--'),
                 lines.Line2D([0], [0], color = handles[4].get_color(), linestyle = '-')]
leg_strat = ax.legend(handles_strat, ['tmim', 'teoo'], loc = 'lower left',
                      frameon = False, bbox_to_anchor = (0.38, -0.368))
ax.add_artist(leg_strat)


handles_mhw = [lines.Line2D([0], [0], color = 'k', linestyle = '-'),
           lines.Line2D([0], [0], color = 'k', linestyle = '--')]
ax.legend(handles_mhw, ['0 m NHN', 'MHW = 1.25 m NHN'], 
          loc = 'lower right', frameon = False,
          bbox_to_anchor = (1.01, -0.02))

fig.savefig('Fig6.png', bbox_inches = 'tight', dpi = fig.dpi)
